module Enginery
  class Registry
    include Helpers

    def initialize root
      @dst_root = root.freeze
    end

    def controllers
      boot_app
      app_controllers.inject({}) do |f,c|
        path   = controller_path(c)
        routes = routes(c)
        data   = {
            name: c.name,
            path: path,
            file: unrootify(dst_path(:controllers, path + CONTROLLER_SUFFIX)),
          dom_id: c.name.gsub(/\W/m, ''),
          routes: routes,
           specs: routes.inject(0) {|s,(_,r)| s += r[:specs].size}
        }
        helper_file = dst_path(:helpers, path + HELPER_SUFFIX)
        File.exists?(helper_file) && data[:helper_file] = unrootify(helper_file)
        f.merge c.name => data
      end.to_yaml
    end

    def routes controller
      path = controller_path(controller)
      routes_by_controller(controller.name).inject({}) do |f,r|
        f.merge r => {
          name:  r,
          route: controller[r.to_sym],
          file:  unrootify(dst_path(:controllers, path, r + ROUTE_SUFFIX)),
          views: views(controller, r),
          specs: specs(controller, r)
        }
      end
    end

    def views controller, route
      path = controller_path(controller)
      Dir[dst_path(:views, path, route + '.*')].map {|f| unrootify(f)}
    end

    def specs controller, route
      path = controller_path(controller)
      Dir[dst_path(:specs, path, route + SPEC_SUFFIX)].map {|f| unrootify(f)}
    end

    def models
      cfg = app_config()
      app_models.inject({}) do |f,m|
        path = model_path(m)
        f.merge m.name => {
                 name: m.name,
                 path: path,
                 file: unrootify(dst_path(:models, path + MODEL_SUFFIX)),
            rear_file: rear_file(m),
            rear_path: rear_path(m, cfg),
               dom_id: m.name.gsub(/\W/m, ''),
                  orm: cfg[:orm],
           migrations: migrations(m)
        }
      end.to_yaml
    end

    def migrations model
      migrations_by_model(model.name).inject({}) do |f,m| 
        step, time, name = m.scan(Migrator::NAME_REGEXP).flatten
        file = dst_path(:migrations, model_path(model), m)
        f.merge ('%s. %s' % [step, name.to_s.gsub('-', ' ')]) => {
          step: step,
          name: name,
          time: time,
          file: unrootify(file),
          path: file.sub(dst_path.migrations, '')
        }
      end
    end

    private
    def controller_path controller
      EUtils.class_to_route(controller.name)
    end

    def model_path model
      EUtils.class_to_route(model.name)
    end

    def rear_path model, cfg
      return unless admin_url = cfg[:admin_url]
      EUtils.rootify_url(admin_url, EUtils.class_to_route(model.name))
    end

    def rear_file model
      unrootify dst_path(:rear_controllers, model_path(model) + ADMIN_SUFFIX)
    end

  end
end
